/**
 * Mysql diver instance.
 * @memberof module:pon-db-mysql-driver/lib
 * @function resource
 * @param {object} connection - Mysql connection
 * @param {string} name - Resource name.
 * @param {object} options - Optional settings.
 * @returns {object} Resource instance.
 */

"use strict";

var ponDbDriver = require('pon-db-driver'),
    ponSql = require('pon-sql'),
    async = require('async'),
    _clean = require('./_clean'),
    _format = require('./_format');

/** @lends resource */
function resource(connection, tableName, options) {
    options = options || {};

    var sql = ponSql(tableName, {
        idKey: options.idKey
    });

    function executeSql(sql, callback) {
        connection.query({
            sql: sql,
            nestTables: '.'
        }, function (err, result) {
            callback(err, _format(tableName, result));
        });
    }

    var resource = ponDbDriver.resource({
        create: function create(data, callback) {
            data = _clean(data);
            async.waterfall([
                function (callback) {
                    executeSql(sql.create(data), callback);
                },
                function (result, callback) {
                    resource.one(result.insertId, callback);
                },
                function (data, callback) {
                    callback(null, {
                        success: true,
                        data: data
                    });
                }
            ], callback);
        },
        update: function update(id, data, callback) {
            data = _clean(data);
            async.waterfall([
                function (callback) {
                    executeSql(sql.update(id, data), callback);
                },
                function (result, callback) {
                    resource.one(id, callback);
                },
                function (data, callback) {
                    callback(null, {
                        success: true,
                        data: data
                    });
                }
            ], callback);
        },
        updateBulk: function updateBulk(condition, data, callback) {
            data = _clean(data);
            async.waterfall([
                function (callback) {
                    executeSql(sql.updateBulk(condition, data), callback);
                },
                function (result, callback) {
                    resource.list({
                        condition: condition
                    }, callback);
                },
                function (data, callback) {
                    callback(null, {
                        success: true,
                        data: data
                    });
                }
            ], callback);
        },
        destroy: function destroy(id, callback) {
            async.waterfall([
                function (callback) {
                    resource.one(id, callback);
                },
                function (data, callback) {
                    executeSql(sql.destroy(id), function done(err) {
                        callback(err, {
                            success: true,
                            data: data
                        });
                    });
                }
            ], callback);
        },
        destroyBulk: function destroyBulk(condition, callback) {
            async.waterfall([
                function (callback) {
                    resource.list({
                        condition: condition
                    }, callback);
                },
                function (data, callback) {
                    executeSql(sql.destroyBulk(condition), function done(err) {
                        callback(err, {
                            success: true,
                            data: data
                        });
                    });
                }
            ], callback);
        },
        one: function one(id, callback) {
            executeSql(sql.one(id), function done(err, result) {
                callback(err, result && result.shift());
            });
        },
        list: function list(config, callback) {
            executeSql(sql.list(config), function done(err, result) {
                callback(err, result);
            });
        },
        count: function count(conifg, callback) {
            executeSql(sql.count(conifg), function done(err, result) {
                callback(err, result && result.shift()['count']);
            });
        },
        define: function (model, callback) {
            var sql = model.toSqliteSql();
            executeSql(sql, callback);
        },
        drop: function (callback) {
            var sql = 'DROP TABLE IF EXISTS ' + tableName;
            executeSql(sql, callback);
        }
    });
    return resource;
}


resource._format = _format;
module.exports = resource;